/*******************************************************************************
 * Licensed Materials - Property of IBM
 * (c) Copyright IBM Corporation 2007-2008. All Rights Reserved.
 * 
 * Note to U.S. Government Users Restricted Rights:
 * Use, duplication or disclosure restricted by GSA ADP Schedule
 * Contract with IBM Corp.
 *******************************************************************************/
(function() {

dojo.provide("com.ibm.rational.report.web.core.app.view.admin.EditContributersDialog");

dojo.require("dijit._Widget");
dojo.require("dijit._Templated");
dojo.require("dojox.data.dom");
dojo.require("com.ibm.rational.report.web.core.app.internal.MessageFormatter");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.ErrorDialog");
dojo.require("com.ibm.rational.report.web.core.client.internal.UserClient");
dojo.require("com.ibm.rational.report.web.core.client.internal.ProjectClient");
dojo.require("com.ibm.team.repository.web.transport.ServiceResponseHandler");
dojo.require("com.ibm.rational.report.web.core.app.internal.common.CommonDialog");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
dojo.requireLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");

var CommonMessages	    	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.internal.common", "Messages");
var ServiceabilityMessages	= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.serviceability", "Messages");
var AdminMessages			= dojo.i18n.getLocalization("com.ibm.rational.report.web.core.app.view.admin", "Messages");
var MessageFormatter		= com.ibm.rational.report.web.core.app.internal.MessageFormatter;
var UserClient				= com.ibm.rational.report.web.core.client.internal.UserClient;
var ProjectClient			= com.ibm.rational.report.web.core.client.internal.ProjectClient;
var ServiceResponseHandler	= com.ibm.team.repository.web.transport.ServiceResponseHandler;

// Most Recently used values
var MRU 					= new Object();
MRU.regexp					= "";

/**
 * Displays a dialog to select users to grant access to the website.
 */
dojo.declare(
	"com.ibm.rational.report.web.core.app.view.admin.EditContributersDialog",
	[dijit._Widget, dijit._Templated],	{
	
	templatePath: dojo.moduleUrl("com.ibm.rational.report.web.core", "app/view/admin/templates/EditContributersDialog.html"),
	
	constructor: function(args) {
		this._controller = args.controller;
		this._viewer = args.viewer;
	},
	
	postCreate: function() {
		this._initText();
				
		this.allUsers.checked = true;
		this._projectArea = com.ibm.team.process.web.client.projectarea.getCurrentProjectArea();
		this._handleChoice();
		this._msgFormatter = new MessageFormatter(this.messageArea);
		
		var self = this;
		
		this.searchButton.disabled = true;
		
		dojo.connect(this.searchButton, "onclick",
			function(e) {
				var name = dojo.trim(self.searchUsers.value);
				self.populateContributers(name);
				self.okButton.disabled = true;
			}
		);
		
		dojo.connect(this.searchUsers, "onkeyup",
			function(e) {
				e.preventDefault();
				var name = dojo.trim(self.searchUsers.value);
				self.searchButton.disabled = (name.length == 0);
				MRU.regexp = name;
			}
		);
		
		dojo.connect(this.searchUsers, "onfocus",
			function(e) {
				MRU.regexp = self.searchUsers.value;
				self.searchUsers.value = "";
			}
		);
		
		dojo.connect(this.searchUsers, "onblur",
			function(e) {
				if(self.searchUsers.value == ""){
					if(MRU.regexp == "")
						self.searchUsers.value = CommonMessages.TYPE_TO_FILTER ;
					else	
						self.searchUsers.value = MRU.regexp;
				}
			}
		);
							
		this._refreshUserRegistry();
		
		this.dialog = new com.ibm.rational.report.web.core.app.internal.common.CommonDialog({
			contentNode: this.contentNode,
			primaryTitle: CommonMessages.ADD_USERS_DIALOG_TITLE,
			width: (dojo.isIE) ? "372px" : "354px",
			isClosable: true
		});	
		this.cancelButton.focus();
	},
	
	_initText: function() {
		dojox.data.dom.removeChildren(this.clearQuestUsersLabel);
		this.clearQuestUsersLabel.appendChild(document.createTextNode(AdminMessages.ALL_USERS));
		
		dojox.data.dom.removeChildren(this.specificUsersLabel);
		this.specificUsersLabel.appendChild(document.createTextNode(AdminMessages.SPECIFIC_USERS));
		
		dojox.data.dom.removeChildren(this.okButton);
		this.okButton.appendChild(document.createTextNode(CommonMessages.OK_BUTTON_LABEL));
		
		dojox.data.dom.removeChildren(this.cancelButton);
		this.cancelButton.appendChild(document.createTextNode(CommonMessages.CLOSE_BUTTON_LABEL));
		
		dojox.data.dom.removeChildren(this.searchButton);
		this.searchButton.appendChild(document.createTextNode(CommonMessages.SEARCH_BUTTON_LABEL));
		
		this.searchUsers.value = CommonMessages.TYPE_TO_FILTER;
		
		dojox.data.dom.removeChildren(this.dialogText);
		this.dialogText.appendChild(document.createTextNode(CommonMessages.ADD_USERS_TO_GRANT_ACCESS));
	},
	
    _handleChoice: function(){
		this.searchUsers.disabled = this.allUsers.checked;
		this.searchButton.disabled = this.allUsers.checked;
		this._setUserTable(this.allUsers.checked);
		this.okButton.disabled = !this.allUsers.checked;	
		if (!this.allUsers.checked) {
			if(MRU.regexp == ""){
				this.searchUsers.value = CommonMessages.TYPE_TO_FILTER;
			}
			else{
				this.searchUsers.value = MRU.regexp;
			}
			var name = dojo.trim(this.searchUsers.value);
			this.searchButton.disabled = (name.length == 0);
			var self = this;
			var inputs = this.usersTableBody.getElementsByTagName("input");
			for (var i = 0; i < inputs.length; i++) {
				if (inputs[i].checked) {
					self.okButton.disabled = false;
					break;
				}
			}
		}
		else{
			this.searchUsers.value = CommonMessages.TYPE_TO_FILTER;
		}
	},
	
  	_deleteUserTable: function() {
		var rows = this.usersTable.tBodies[0].getElementsByTagName("tr");
		var i =0;

		while(this.usersTable.tBodies[0].getElementsByTagName("tr").length > 0){
			if (this.usersTable.tBodies[0].getElementsByTagName("tr").length > 0) {
				this.usersTable.tBodies[0].deleteRow(0);
			}
		}
	},
	
	_setUserTable : function(readOnly) {
		var length = this.usersTableForm.length;
		for (var i = 0; i < length; i++) {
			this.usersTableForm.elements[i].disabled = readOnly;
		}
	},
	
	addContributors: function() {
		var userNames = new Array();
		if (this.allUsers.checked == true) {
			// passing empty, would find all the non contributers 
			// and make them contributers with one server call
			this._addContributors(userNames);
		}
		else {
			var inputs = this.usersTableBody.getElementsByTagName("input");
			var userNames = new Array();
			dojo.forEach(inputs, function(input){
				if (input.checked) {
					userNames.push(input.id);
				}
			});
			if (userNames.length > 0)
				this._addContributors(userNames);
		}
	},
	
	_addContributors: function(userNames){
		var srh = new ServiceResponseHandler(this, "_addAllUsersSuccess", "_addAllUsersError");		
		UserClient.postNewContributors(srh, {names: userNames});			
	},
	
	populateContributers: function(pattern){
		this._msgFormatter.displayLoadingMessage(AdminMessages.USER_ADMIN_RETRIEVING_USERS_FROM_SERVER);
		this._deleteUserTable();
		var responseHandler = new ServiceResponseHandler(this, "_displayUsers", "_getUsersError");				
		UserClient.getUsers(responseHandler, {namePattern: pattern, nonContributersOnly: "true"});	
	},
	
	_displayUsers: function(users) {
		if (users != null && users.length != 0) {
			for (var i = 0; i < users.length; i++) {
				this._addOption(users[i].name);
			}
			this._msgFormatter.displayNoMessage();
		}
		else {
			this._msgFormatter.displayErrorMessage(ServiceabilityMessages.NO_NEW_MATCHING_USERS_FOUND);
		}
	},
	
	_addOption: function(userName){
		var self = this;
		var row = document.createElement("tr");
		this.usersTableBody.appendChild(row);
		var td = row.insertCell(0);
		var input = document.createElement("input");
		input.type = "checkbox";
        input.onclick = function(){
            var result = true;
            var inputs = self.usersTableBody.getElementsByTagName("input");
            dojo.forEach(inputs, function(input){
                if (input.checked) {
                    result = false;
                }
            });
            self.okButton.disabled = result;
        };
		input.id = userName;
		td.appendChild(input);
		td = row.insertCell(1);
		var text = document.createTextNode(userName);
		td.appendChild(text);
	},
	
	_getUsersError : function() {
		this._displayErrorMessage(ServiceabilityMessages.FAILED_TO_RETRIEVE_USERS);
	},
	
	_addAllUsersError: function(){
		this._viewer.clearMessageArea();
		this._displayErrorMessage(ServiceabilityMessages.FAILED_TO_GRANT_SELECTED_USERS_ACCESS_TO_WEBSITE);
	},
	
	_addAllUsersSuccess: function(){
		this._controller.manageUsers();
	},
	
	_refreshUserRegistry: function(e) {	
		this._msgFormatter.displayLoadingMessage(ServiceabilityMessages.USER_REGISTRY_BEING_REFRESHED);
		var srh = new ServiceResponseHandler(this, "_refreshSuccess", "_refreshError");				
		UserClient.postRefreshUserRegistry(srh);
	},
	
	_refreshSuccess: function() {
		this._msgFormatter.displayNoMessage();		
	},
	
	_refreshError: function(error) {
		this._msgFormatter.displayErrorMessage(ServiceabilityMessages.FAILED_TO_REFRESH_USER_REGISTRY + error);
	},
	
	_displayErrorMessage: function(errorMsg){
		var errorDialog = new com.ibm.rational.report.web.core.app.internal.common.ErrorDialog();
		errorDialog.displayError(errorMsg);
	},
	
	close: function() {
		this.dialog.close();
	}
	
});

})();